#include <stdio.h>

#define BOARD_SIZE     8
#define COLUMN_LAYOUTS 55 // Must be (BOARD_SIZE + 2)-th Fibonacci number.
#define MOD            1000000007

typedef unsigned long long int LLU;

LLU D[COLUMN_LAYOUTS][COLUMN_LAYOUTS], T1[COLUMN_LAYOUTS][COLUMN_LAYOUTS], T2[COLUMN_LAYOUTS][COLUMN_LAYOUTS];

void mul(LLU T1[][COLUMN_LAYOUTS], LLU T2[][COLUMN_LAYOUTS], LLU T3[][COLUMN_LAYOUTS])
{
    int i, j, k;

    for (i = 0; i < COLUMN_LAYOUTS; i++)
    {
        for (j = 0; j < COLUMN_LAYOUTS; j++)
        {
            T3[i][j] = 0;
            for (k = 0; k < COLUMN_LAYOUTS; k++)
            {
                T3[i][j] += (T1[i][k] * T2[k][j]) % MOD;
                T3[i][j] %= MOD;
            }
        }
    }
}

void mxp(LLU T1[][COLUMN_LAYOUTS], LLU T2[][COLUMN_LAYOUTS], LLU T3[][COLUMN_LAYOUTS], LLU N)
{
    int i, j, k;

    if (N == 0)
    {
        for (i = 0; i < COLUMN_LAYOUTS; i++)
            for (j = 0; j < COLUMN_LAYOUTS; j++)
                T2[i][j] = 0;

        for (i = 0; i < COLUMN_LAYOUTS; i++)
            T2[i][i] = 1;
    }
    else if (N == 1)
    {
        for (i = 0; i < COLUMN_LAYOUTS; i++)
            for (j = 0; j < COLUMN_LAYOUTS; j++)
                T2[i][j] = T1[i][j];
    }
    else if (N & 1)
    {
        /* T2 = T1^(N/2) */
        mxp(T1, T2, T3, N / 2);
        /* T3 = T2 * T2 = T1^(N - 1) */
        mul(T2, T2, T3);
        /* T2 = T1 * T3 = T1^N */
        mul(T1, T3, T2);
    }
    else
    {
        /* T3 = T1^(N/2) */
        mxp(T1, T3, T2, N / 2);

        /* T2 = T3 * T3 = T1^N */
        mul(T3, T3, T2);
    }
}

int main(void)
{
    int i, j, k, l;
    LLU M, ans = 0;

    scanf("%llu", &M);

    if (M == 0)
    {
        printf("0\n");
        return;
    }

    k = -1;
    for (i = 0; i < (1 << BOARD_SIZE); i++)
    {
        if ((i & (i >> 1)) == 0)
        {
            ++k;

            l = -1;
            for (j = 0; j < (1 << BOARD_SIZE); j++)
            {
                if ((j & (j >> 1)) == 0)
                {
                    ++l;

                    if ((i & j) == 0)
                    {
                        D[k][l] = D[l][k] = 1;
                    }
                }
            }
        }
    }

    mxp(D, T1, T2, M - 1);

    for (i = 0; i < COLUMN_LAYOUTS; i++)
    {
        for (j = 0; j < COLUMN_LAYOUTS; j++)
        {
            ans += T1[i][j];
            ans %= MOD;
        }
    }

    printf("%lld\n", ans);

    return 0;
}
